-- This software is distributed under the terms of the new BSD License.
-- Copyright (c) 2014 molelord
-- All rights reserved.

-- TD4(コアのみ)向けのテストベンチ

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.compo_pkg.all;
use work.td4inst_pkg.all;

entity td4core_tb is
end td4core_tb;

architecture sim of td4core_tb is

    signal clk      : std_logic := '0';
    signal rst_n    : std_logic := '1';
    signal data     : std_logic_vector(7 downto 0);
    signal addr     : std_logic_vector(3 downto 0);
    signal iport    : std_logic_vector(3 downto 0) := (others=>'0');
    signal oport    : std_logic_vector(3 downto 0);

    subtype romline is std_logic_vector(7 downto 0);
    type rom is array (0 to 15) of romline;
    constant INSTROM : rom := (
        OUT_IM   & "0011", -- 0
        OUT_IM   & "0110", -- 1
        OUT_IM   & "1100", -- 2
        OUT_IM   & "1000", -- 3
        OUT_IM   & "1000", -- 4
        OUT_IM   & "1100", -- 5
        OUT_IM   & "0110", -- 6
        OUT_IM   & "0011", -- 7
        OUT_IM   & "0001", -- 8
        JMP_IM   & "0000", -- 9
        ADD_A_IM & "0000", -- a
        ADD_A_IM & "0000", -- b
        ADD_A_IM & "0000", -- c
        ADD_A_IM & "0000", -- d
        ADD_A_IM & "0000", -- e
        ADD_A_IM & "0000"  -- f
    );

    constant HALFCYCLE : time := 50 ms; -- 10Hz

begin

    clock : process
    begin
        wait for HALFCYCLE;
        while true loop
            wait for HALFCYCLE;
            clk <= not clk;
        end loop;
    end process;

    reset : process
    begin
        for i in 1 to 3 loop
            wait for HALFCYCLE;
        end loop;
        rst_n <= '0';
        wait for HALFCYCLE;
        wait for HALFCYCLE;
        rst_n <= '1';
        wait;
    end process;

    data <= INSTROM(to_integer(unsigned(addr)));

    dut : td4core
    port map (
        clock_i   => clk,
        reset_n_i => rst_n,
        ce_i      => '1',
        d_i       => data,
        a_o       => addr,
        iport_i   => iport,
        oport_o   => oport
    );

end sim;
